"""DTML TeX objects."""

__version__='0.01'

from Globals import HTML, HTMLFile, MessageDialog
from OFS.content_types import guess_content_type
from OFS.DTMLMethod import DTMLMethod, decapitate
from AccessControl.User import verify_watermark
from urllib import quote

class DTMLTeX( DTMLMethod ):
    """DTML TeX objects are DTML-Methods that produce Postscript from TeX"""
    meta_type='DTML TeX'
    
    index_html=None # Prevent accidental acquisition

    manage_options=({'label':'Edit', 'action':'manage_main'},
                    {'label':'Upload', 'action':'manage_uploadForm'},
                    {'label':'View', 'action':''},
                    {'label':'Proxy', 'action':'manage_proxyForm'},
                    {'label':'Security', 'action':'manage_access'},
                   )
    __ac_permissions__=(
    ('View management screens',
     ('manage_editForm', 'manage', 'manage_main', 'manage_uploadForm',
      'document_src', 'PrincipiaSearchSource')),
    ('Change DTML Methods',     ('manage_edit', 'manage_upload', 'PUT')),
    ('Change proxy roles', ('manage_proxyForm', 'manage_proxy')),
    ('View', ('__call__', '')),
    ('FTP access', ('manage_FTPstat','manage_FTPget','manage_FTPlist')),
    )

    def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
        """Render the document given a client object, REQUEST mapping,
        Response, and key word arguments."""
	
	#this list takes the temporary-file objects
        tmp = [] 
        
	kw['document_id']   =self.id
        kw['document_title']=self.title
	kw['__temporary_files__'] =tmp
	
	
        # Verify the authenticated user object.
        if REQUEST.has_key('AUTHENTICATED_USER'):
            verify_watermark(REQUEST['AUTHENTICATED_USER'])
	
	if client is None:
            # Called as subtemplate, so don't need error propigation!
            # Give a reference to this only
            r = "<a href='" + self.__name__ + "'>" + str(self.title) + "</a>"
            return r
            
        #Resovle DTML, make Postscript from TeX
        r= apply(HTML.__call__, (self, client, REQUEST), kw)
        r= latex( r )
        r="content-type: application/postscript\n\n" + r
        
	if type(r) is not type(''): return r
                
        if RESPONSE is None: return r

        # Ick.  I don't like this. But someone can override it with
        # a header if they have to.
        hh=RESPONSE.headers.has_key
        if not (hh('content-type') or hh('Content-Type')):
            c, e=guess_content_type(self.__name__, r)
            RESPONSE.setHeader('Content-Type', c)
        return decapitate(r, RESPONSE)

default_dm_html="""\\documentclass{article}
\\begin{document}
\\end{document}
"""

addForm=HTMLFile('texAdd', globals())

def add(self, id, title='', file='', REQUEST=None, submit=None):
    """Add a DTML TeX object with the contents of file. If
    'file' is empty, default document text is used.
    """
    if type(file) is not type(''): file=file.read()
    if not file: file=default_dm_html
    ob=DTMLTeX(file, __name__=id)
    ob.title=title
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''

#### This is for running the latex-command
from os import chdir, execv, fork, wait, unlink
from thread import start_new_thread
from time import sleep
from tempfile import mktemp
from glob import glob
import tempfile

#global constants, change this for your desire
tempfile.tempdir='/tmp'
LATEX='/usr/bin/latex'
DVIPS='/usr/bin/dvips'

def tmpcmd ( path, args ):
    pid= fork()
    if pid==0:
	#we are the child
	chdir( tempfile.tempdir )
	execv( path, args )
    elif pid<0:
        #something goes wrong
        raise 'cant fork for command'
    if wait( pid )[1]:
        raise 'nonzero exitstatus from child'
    return

def latex( data ):
    try:
        base = mktemp()
        tex  = base + ".tex"
        dvi  = base + ".dvi"
        ps   = base + ".ps"
        
        # create temporary tex file
        f= open( tex, "w" )
        f.write( data )
        f.close()
        
        tmpcmd( LATEX, ('latex','-interaction=batchmode',tex) )
        tmpcmd( DVIPS, ('dvips','-q', '-o', ps, dvi ) )
        
        f  = open( ps, "r" )
        out= f.read()
        f.close()
        
    finally:
        for i in glob(base+".*"):
            unlink( i )
    return out

### Extension to File class follows
import OFS.Image

def delete_tempfile_thread( tmp, t ):
    sleep( t )
    unlink( tmp )
    return

def create_temp( self , t=60, sfx=0 ):
    if hasattr( self, 'suffix' ):
        suffix= self.suffix
    else:
        suffix= ''
    base = mktemp()
    tmp  = base + suffix
    f= open( tmp, "w" )
    data= self.data
    if type(data) is not type(''):
        while data:
            f.write(data.data)
            data= data.next
    else:
        f.write( data )
    f.close()
    #this removes the temporary file in t seconds
    start_new_thread( delete_tempfile_thread, (tmp,t) )
    if sfx:
        return tmp
    else:
        return base

OFS.Image.File.create_temp= create_temp

